@@ -1,4 +1,6 @@
 // basic headers
+#define _GNU_SOURCE /* really just for strndup */
+
 #include <stdio.h>
 
 #ifdef HAVE_STDLIB_H
@@ -47,10 +49,14 @@
 
 // this doesn't really belong here, but it was easiest
 #ifdef HAVE_MAGICK
-#define BUILDSPEC_MAGICK " magick"
+#define BUILDSPEC_MAGICK " imagemagick"
+#else
+#ifdef HAVE_GMAGICK
+#define BUILDSPEC_MAGICK " graphicsmagick"
 #else
 #define BUILDSPEC_MAGICK ""
 #endif
+#endif
 
 #ifdef HAVE_GETOPT_LONG
 #define BUILDSPEC_GETOPT " gnugetopt"
@@ -78,10 +84,20 @@
 
 #define BUILDSPEC BUILDSPEC_GETOPT BUILDSPEC_MAGICK BUILDSPEC_ICONV BUILDSPEC_FREETYPE BUILDSPEC_FRIBIDI
 
+unsigned int strtounsigned
+  (
+    const char * s,
+    const char * what /* description of what I'm trying to convert, for error message */
+  );
+  /* parses s as an unsigned decimal integer, returning its value. Aborts the
+    program on error. */
 
-
-#ifndef HAVE_STRSEP
-char *strsep(char **stringp,const char *delim);
+#ifndef HAVE_STRNDUP
+char * strndup
+  (
+    const char * s,
+    size_t n
+  );
 #endif
 
 #if !HAVE_DECL_O_BINARY
@@ -96,20 +112,34 @@ char *strsep(char **stringp,const char *delim);
 
 #define PACKAGE_HEADER(x) PACKAGE_NAME "::" x ", version " PACKAGE_VERSION ".\nBuild options:" BUILDSPEC "\nSend bugs to <" PACKAGE_BUGREPORT ">\n\n"
 
-#ifdef ICONV_CONV
-#define ICONV_CAST (const char **)
-#else
-#define ICONV_CAST (char **)
-#endif
-
 #ifndef HAVE_FT2BUILD_H
 #define FT_FREETYPE_H <freetype/freetype.h>
 #define FT_GLYPH_H <freetype/ftglyph.h>
 #endif
 
-struct vfile {
-    FILE *h;
-    int ftype;
-};
-extern struct vfile varied_open(const char *fname,int mode);
-extern void varied_close(struct vfile vf);
+/* values for vfile.ftype */
+#define VFTYPE_FILE 0 /* an actual file I opened */
+#define VFTYPE_PIPE 1 /* an actual pipe I opened to/from a child process */
+#define VFTYPE_REDIR 2 /* a redirection to/from another already-opened file */
+struct vfile /* for keeping track of files opened by varied_open */
+  {
+    FILE * h; /* do your I/O to/from this */
+    int ftype, mode; /* for use by varied_close */
+  } /*vfile*/;
+struct vfile varied_open
+  (
+    const char * fname,
+    int mode, /* either O_RDONLY or O_WRONLY, nothing more */
+    const char * what /* description of what I'm trying to open, for error message */
+  );
+  /* opens the file fname, which can be an ordinary file name or take one of the
+    following special forms:
+        "-" -- refers to standard input (if mode is O_RDONLY) or output (if O_WRONLY)
+        "&n" -- (n integer) refers to the already-opened file handle n
+        "cmd|" -- spawns cmd as a subprocess and reads from its standard output
+        "|cmd" -- spawns cmd as a subprocess and writes to its standard input.
+
+    Will abort the process on any errors.
+  */
+void varied_close(struct vfile vf);
+  /* closes a file previously opened by varied_open. */
