@@ -1,3 +1,7 @@
+/*
+    dvdauthor mainline, interpretation of command-line options and parsing of
+    dvdauthor XML control files
+*/
 /*
  * Copyright (C) 2002 Scott Smith (trckjunky@users.sourceforge.net)
  *
@@ -30,63 +34,108 @@
 #include "readxml.h"
 #include "rgb.h"
 
+static char * str_extract_until
+  (
+    const char ** src,
+    const char * delim
+  )
+  /* scans *src, looking for the first occurrence of a character in delim. Returns
+    a copy of the prior part of *src if found, and updates *src to point after the
+    delimiter character; else returns a copy of the whole of *src, and sets *src
+    to NULL. Returns NULL iff *src is NULL. */
+  {
+    char * result = NULL; /* to begin with */
+    if (*src != NULL)
+      {
+        const size_t pos = strcspn(*src, delim);
+        if (pos < strlen(*src))
+          {
+            result = strndup(*src, pos);
+            *src = *src + pos + strspn(*src + pos, delim);
+          }
+        else
+          {
+            result = strdup(*src);
+            *src = NULL;
+          } /*if*/
+      } /*if*/
+    return
+        result;
+  } /*str_extract_until*/
+
+/* common parsing bits for both command line and XML file */
 
 #define RGB2YCrCb(R,G,B) ((((int)RGB2Y(R,G,B))<<16)|(((int)RGB2Cr(R,G,B))<<8)|(((int)RGB2Cb(R,G,B))))
 
-static int readdvdauthorxml(const char *xmlfile,char *fb);
+static int readdvdauthorxml(const char *xmlfile,const char *fb);
 
-static int hadchapter=0,pauselen=0,writeoutput=1;
-static char *chapters=0;
+static enum
+  {
+    chapters_neither, /* neither of following seen yet */
+    chapters_cells, /* vob chapters specified via cells (and possibly also "chapters" attribute) */
+    chapters_chapters, /* only via "chapters" attribute */
+  }
+    hadchapter = chapters_neither; /* info about last VOB in current PGC */
+static int
+    pauselen=0,
+    writeoutput=1;
+static char
+    *chapters=0;
 
-static void parsevideoopts(struct pgcgroup *va,char *o)
+static void parsevideoopts(struct pgcgroup *va,const char *o)
 {
     char *s;
-    while(NULL!=(s=strsep(&o,"+"))) {
+    while(NULL!=(s=str_extract_until(&o,"+"))) {
         if(pgcgroup_set_video_attr(va,VIDEO_ANY,s)) {
             fprintf(stderr,"ERR:  Video option '%s' overrides previous option\n",s);
             exit(1);
         }
+        free(s);
     }
 }
 
-static void parseaudiotrack(struct pgcgroup *va,char *o,int c)
+static void parseaudiotrack(struct pgcgroup *va,const char *o,int c)
 {
     char *s;
-    while(NULL!=(s=strsep(&o,"+"))) {
+    while(NULL!=(s=str_extract_until(&o,"+"))) {
         if(pgcgroup_set_audio_attr(va,AUDIO_ANY,s,c)) {
             fprintf(stderr,"ERR:  Audio option '%s' on track %d overrides previous option\n",s,c);
             exit(1);
         }
+        free(s);
     }
 }
 
-static void parseaudioopts(struct pgcgroup *va,char *o)
+static void parseaudioopts(struct pgcgroup *va,const char *o)
 {
     char *s;
     int ch=0;
-    while(NULL!=(s=strsep(&o,", "))) {
+    while(NULL!=(s=str_extract_until(&o,", "))) {
         parseaudiotrack(va,s,ch);
+        free(s);
         ch++;
     }
 }
 
-static void parsesubpicturetrack(struct pgcgroup *va,char *o,int c)
+static void parsesubpicturetrack(struct pgcgroup *va,const char *o,int c)
 {
     char *s;
-    while(NULL!=(s=strsep(&o,"+"))) {
+    while(NULL!=(s=str_extract_until(&o,"+"))) {
         if(pgcgroup_set_subpic_attr(va,SPU_ANY,s,c)) {
             fprintf(stderr,"ERR:  Subpicture option '%s' on track %d overrides previous option\n",s,c);
             exit(1);
         }
+        free(s);
     }
 }
 
-static void parsesubpictureopts(struct pgcgroup *va,char *o)
+static void parsesubpictureopts(struct pgcgroup *va,const char *o)
 {
     char *s;
     int ch=0;
-    while(NULL!=(s=strsep(&o,", "))) {
+    while(NULL!=(s=str_extract_until(&o,", "))) {
         parsesubpicturetrack(va,s,ch);
+        free(s);
         ch++;
     }
 }
@@ -104,9 +153,9 @@ static void parsebutton(struct pgc *va,char *b)
     pgc_add_button(va,bnm,b);
 }
 
-static void parseinstructions(struct pgc *va,char *b)
+static void parseinstructions(struct pgc *va,const char *b)
 {
-    char *c=strsep(&b,"=");
+    char *c=str_extract_until(&b,"=");
     if(!strcasecmp(c,"post")) {
         pgc_set_post(va,b);
     } else if(!strcasecmp(c,"pre")) {
@@ -115,17 +164,21 @@ static void parseinstructions(struct pgc *va,char *b)
         fprintf(stderr,"Unknown instruction block: %s\n",c);
         exit(1);
     }
+    free(c);
 }
 
-static void parseentries(struct pgc *p,char *b)
+static void parseentries(struct pgc *p,const char *b)
 {
     char *v;
 
-    while(NULL!=(v=strsep(&b,", ")))
+    while(NULL!=(v=str_extract_until(&b,", ")))
+      {
         pgc_add_entry(p,v);
+        free(v);
+      } /*while*/
 }
 
-static double parsechapter(char *s)
+static double parsechapter(const char *s)
 {
     double total=0,field=0;
     int i;
@@ -152,35 +205,36 @@ static double parsechapter(char *s)
     return total*60+field;
 }
 
-static void parsechapters(char *o,struct source *src,int pauselen)
-{
+static void parsechapters(const char *o, struct source *src, int pauselen)
+  /* parses a chapter spec and adds corresponding cell definitions. */
+  {
     char *s;
-    double last=0;
-    int lastchap=0;
-
-    while(NULL!=(s=strsep(&o,", "))) {
-        double total=parsechapter(s);
-        if( total>last ) {
-            source_add_cell(src,last,total,lastchap,0,0);
-            last=total;
-            lastchap=1;
-        } else if( total==last )
-            lastchap=1;
-    }
-    source_add_cell(src,last,-1,lastchap,pauselen,0);
-}
+    double last = 0;
+    cell_chapter_types lastchap = CELL_NEITHER;
+    while (NULL != (s = str_extract_until(&o, ", ")))
+      {
+        const double total = parsechapter(s);
+        if (total > last)
+          {
+            source_add_cell(src, last, total, lastchap, 0, 0);
+            last = total;
+            lastchap = CELL_CHAPTER_PROGRAM;
+          }
+        else if (total == last)
+            lastchap = CELL_CHAPTER_PROGRAM;
+      /* else report error? user specified chapter times in wrong order? */
+        free(s);
+      } /*while*/
+    source_add_cell(src, last, -1, lastchap, pauselen, 0);
+  } /*parsechapters*/
 
 static void readpalette(struct pgc *p,const char *fname)
 {
     int i,rgbf;
     struct vfile h;
 
-    h=varied_open(fname,O_RDONLY);
+    h=varied_open(fname, O_RDONLY, "palette file");
     
-    if( !h.h ) {
-        fprintf(stderr,"ERR:  Cannot open palette file '%s'\n",fname);
-        exit(1);
-    }
     /* write out colors, the hex is the 0yuv combined in one integer 00yyuuvv */
     i=strlen(fname);
     rgbf=( i>=4 && !strcasecmp(fname+i-4,".rgb") );
@@ -188,7 +242,7 @@ static void readpalette(struct pgc *p,const char *fname)
         int pcolor;
         fscanf(h.h, "%x", &pcolor);
         if( rgbf ) {
-            int r=(pcolor>>16)&255,
+            const int r=(pcolor>>16)&255,
                 g=(pcolor>>8)&255,
                 b=(pcolor)&255;
             pcolor=RGB2YCrCb(r,g,b);
@@ -291,6 +345,7 @@ static void usage()
     exit(1);
 }
 
+/* bits for authoring via command line */
 #define MAINDEF                                                      \
             if( !usedtocflag ) {                                     \
                 fprintf(stderr,"ERR:  Must first specify -t, -m, or -x.\n"); \
@@ -324,7 +379,7 @@ int main(int argc,char **argv)
 {
     struct pgcgroup *va[2];
     struct menugroup *mg;
-    char *fbase=0;
+    char *fbase=0; /* output directory name */
     int curva=1,istoc=0,l,
         usedtocflag=0; // whether the 'istoc' value has been used or not
     struct pgc *curpgc=0,*fpc=0;
@@ -370,7 +425,7 @@ int main(int argc,char **argv)
         switch(c) {
         case 'h':
             usage();
-            break;
+        break;
 
         case 'x':
             if( usedtocflag ) {
@@ -381,15 +436,15 @@ int main(int argc,char **argv)
 
         case 'n':
             writeoutput=0;
-            break;
+        break;
 
         case 'j':
             dvdauthor_enable_jumppad();
-            break;
+        break;
 
         case 'g':
             dvdauthor_enable_allgprm();
-            break;
+        break;
 
         case 'T':
             if( usedtocflag ) {
@@ -398,18 +453,18 @@ int main(int argc,char **argv)
             }
             istoc=1;
             usedtocflag=1; // just for completeness (also see -x FOO)
-            break;
+        break;
 
         case 'o': 
             fbase=optarg;
-            break;
+        break;
 
         case 'm':
             FLUSHPGC;
             usedtocflag=1; // force -T to occur before -m
-            hadchapter=0;
+            hadchapter=chapters_neither; /* reset for new menu */
             curva=0;
-            break;
+        break;
 
         case 't':
             if( istoc ) {
@@ -418,34 +473,34 @@ int main(int argc,char **argv)
             }
             FLUSHPGC;
             usedtocflag=1;
-            hadchapter=0;
+            hadchapter=chapters_neither; /* reset for new title */
             curva=1;
-            break;
+        break;
             
         case 'a':
             MAINDEF;
             parseaudioopts(vc,optarg);
-            break;
+        break;
         
         case 'v':
             MAINDEF;
             parsevideoopts(vc,optarg);
-            break;
+        break;
 
         case 's':
             MAINDEF;
             parsesubpictureopts(vc,optarg);
-            break;
+        break;
 
         case 'b':
             MAINDEFPGC;
             parsebutton(curpgc,optarg);
-            break;
+        break;
 
         case 'i':
             MAINDEFPGC;
             parseinstructions(curpgc,optarg);
-            break;
+        break;
 
         case 'F':
             if( !istoc ) {
@@ -459,7 +514,7 @@ int main(int argc,char **argv)
             }
             fpc=pgc_new();
             pgc_set_pre(fpc,optarg);
-            break;
+        break;
 
         case 'e':
             MAINDEFPGC;
@@ -468,28 +523,41 @@ int main(int argc,char **argv)
                 return 1;
             }
             parseentries(curpgc,optarg);
-            break;
+        break;
 
-        case 'P': optarg=0;
+        case 'P':
+            optarg=0;
+      /* fallthru */
         case 'p':
             MAINDEFPGC;
             readpalette(curpgc,optarg?optarg:"xste-palette.dat");
-            break;
+        break;
 
         case 1:
         case 'f':
             MAINDEFVOB;
             
             source_set_filename(curvob,optarg);
-            if( hadchapter==2 )
-                hadchapter=1;
+            if( hadchapter==chapters_chapters )
+                hadchapter=chapters_cells; /* chapters already specified */
             else
-                source_add_cell(curvob,0,-1,!hadchapter,0,0);
+                source_add_cell
+                  (
+                    /*source =*/ curvob,
+                    /*starttime =*/ 0,
+                    /*endtime =*/ -1,
+                    /*chap =*/ hadchapter == chapters_neither ? CELL_CHAPTER_PROGRAM : CELL_NEITHER,
+                    /*pause =*/ 0,
+                    /*cmd =*/ 0
+                  );
+                  /* default to single chapter for entire source file */
             pgc_add_source(curpgc,curvob);
             curvob=0;
-            break;
+        break;
 
-        case 'C': optarg=0;
+        case 'C':
+            optarg=0;
+      /* fallthru */
         case 'c':
             if( curvob ) {
                 fprintf(stderr,"ERR:  cannot list -c twice for one file.\n");
@@ -497,12 +565,13 @@ int main(int argc,char **argv)
             }
             MAINDEFVOB;
 
-            hadchapter=2;
+            hadchapter=chapters_chapters;
             if( optarg )
                 parsechapters(optarg,curvob,0);
             else
-                source_add_cell(curvob,0,-1,1,0,0);
-            break;
+                source_add_cell(curvob, 0, -1, CELL_CHAPTER_PROGRAM, 0, 0);
+                  /* default to single chapter for entire source file */
+        break;
 
         default:
             fprintf(stderr,"ERR:  getopt returned bad code %d\n",c);
@@ -521,7 +590,7 @@ int main(int argc,char **argv)
     } else
         mg=0;
     if( !va[1] && !istoc )
-        va[1]=pgcgroup_new(0);
+        va[1]=pgcgroup_new(VTYPE_VTS);
 
     if( !fbase && writeoutput ) {
         fbase=readconfentry("WORKDIR");
@@ -553,6 +622,8 @@ int main(int argc,char **argv)
     return 0;
 }
 
+/* authoring via XML file */
+
 enum {
     DA_BEGIN=0,
     DA_ROOT,
@@ -564,63 +635,94 @@ enum {
     DA_NOSUB
 };
 
-static struct pgcgroup *titles=0, *curgroup=0;
-static struct menugroup *mg=0, *vmgmmenus=0;
+static struct pgcgroup
+    *titles=0, /* titles saved here (only one set allowed) */
+    *curgroup=0; /* current menus or titles */
+static struct menugroup
+    *mg=0, /* current menu group (for titleset or vmgm) */
+    *vmgmmenus=0; /* vmgm menu group saved here on completion */
 static struct pgc *curpgc=0,*fpc=0;
 static struct source *curvob=0;
-static char *fbase=0,*buttonname=0;
-static int ismenuf=0,istoc=0,setvideo=0,setaudio=0,setsubpicture=0,subpmode=DA_NOSUB,hadtoc=0,subpstreamid=-1;
-static char *subpstreammode=0;
-static int vobbasic,cell_chapter;
-static double cell_starttime,cell_endtime;
-static char menulang[3];
+static const char
+    *fbase=0, /* output directory name */
+    *buttonname=0; /* name of button currently being defined */
+static vtypes
+    ismenuf = VTYPE_VTS;
+static int
+    istoc=0, /* 1 for vmgm, 0 for titleset */
+    setvideo=0, /* to keep count of <video> tags */
+    setaudio=0, /* to keep count of <audio> tags */
+    setsubpicture=0, /* to keep count of <subpicture> tags */
+    subpmode=DA_NOSUB,
+      /* for determining context of a <subpicture> tag:
+        DA_NOSUB -- invalid
+        DA_PGC -- inside a <pgc> tag (no "lang" attribute allowed)
+        DA_PGCGROUP -- inside a <menus> or <titles> tag, must come before the <pgc> tags
+      */
+    hadtoc=0, /* set to 1 when vmgm seen */
+    subpstreamid=-1; /* id attribute of current <stream> saved here */
+static char *subpstreammode=0; /* mode attribute of current <stream> saved here */
+static enum
+  {
+    vob_has_neither, /* neither of following seen yet */
+    vob_has_chapters_pause, /* vob has "chapters" or "pause" attribute */
+    vob_has_cells, /* vob has <cell> subtags */
+  }
+    vobbasic; /* info about each VOB */
+static cell_chapter_types
+    cell_chapter;
+static double
+    cell_starttime,
+    cell_endtime;
+static char
+    menulang[3];
 
-static void set_video_attr(int attr,char *s)
+static void set_video_attr(int attr,const char *s)
 {
-    if( ismenuf )
-        menugroup_set_video_attr(mg,attr,s);
+    if (ismenuf != VTYPE_VTS)
+        menugroup_set_video_attr(mg, attr, s);
     else
-        pgcgroup_set_video_attr(titles,attr,s);
+        pgcgroup_set_video_attr(titles, attr, s);
 }
 
-static void set_audio_attr(int attr,char *s,int ch)
+static void set_audio_attr(int attr,const char *s,int ch)
 {
-    if( ismenuf )
+    if (ismenuf != VTYPE_VTS)
         menugroup_set_audio_attr(mg,attr,s,ch);
     else
         pgcgroup_set_audio_attr(titles,attr,s,ch);
 }
 
-static void set_subpic_attr(int attr,char *s,int ch)
+static void set_subpic_attr(int attr,const char *s,int ch)
 {
-    if( ismenuf )
+    if (ismenuf != VTYPE_VTS)
         menugroup_set_subpic_attr(mg,attr,s,ch);
     else
         pgcgroup_set_subpic_attr(titles,attr,s,ch);
 }
 
-static void set_subpic_stream(int ch,char *m,int id)
+static void set_subpic_stream(int ch,const char *m,int id)
 {
-    if( ismenuf )
+    if (ismenuf != VTYPE_VTS)
         menugroup_set_subpic_stream(mg,ch,m,id);
     else
         pgcgroup_set_subpic_stream(titles,ch,m,id);
 }
 
-static int parse_pause(char *f)
+static int parse_pause(const char *f)
 {
-    if( !strcmp(f,"inf") )
+    if (!strcmp(f,"inf"))
         return 255;
     else
-        return atoi(f);
+        return strtounsigned(f, "pause time"); /* should check it's in [0 .. 254] */
 }
 
-static void dvdauthor_workdir(char *s)
+static void dvdauthor_workdir(const char *s)
 {
     fbase=utf8tolocal(s);
 }
 
-static void dvdauthor_jumppad(char *s)
+static void dvdauthor_jumppad(const char *s)
 {
     int i=xml_ison(s);
     if( i==1 )
@@ -631,7 +733,7 @@ static void dvdauthor_jumppad(char *s)
     }
 }
 
-static void dvdauthor_allgprm(char *s)
+static void dvdauthor_allgprm(const char *s)
 {
     int i=xml_ison(s);
     if( i==1 )
@@ -656,6 +758,9 @@ static void getfbase()
 }
 
 static void dvdauthor_end()
+/* called on </dvdauthor> end tag, generates the VMGM if specified.
+  This needs to be done after all the titles, so it can include
+  information about them. */
 {
     if( hadtoc ) {
         dvdauthor_vmgm_gen(fpc,vmgmmenus,fbase);
@@ -675,13 +780,13 @@ static void titleset_start()
 }
 
 static void titleset_end()
-{
+{ /* called on </titles> end tag, generates the output titleset. */
     getfbase();
     if( !parser_err ) {
-        if( !titles )
-            titles=pgcgroup_new(0);
+        if (!titles)
+            titles=pgcgroup_new(VTYPE_VTS);
         dvdauthor_vts_gen(mg,titles,fbase);
-        if( mg )
+        if (mg)
             menugroup_free(mg);
         pgcgroup_free(titles);
         mg=0;
@@ -691,7 +796,7 @@ static void titleset_end()
 
 static void vmgm_start()
 {
-    if( hadtoc ) {
+    if (hadtoc) {
         fprintf(stderr,"ERR:  Can only define one VMGM\n");
         parser_err=1;
         return;
@@ -731,217 +836,273 @@ static void fpc_end()
 }
 
 static void pgcgroup_start()
-{
-    setvideo=0;
-    setaudio=-1;
-    setsubpicture=-1;
-    subpmode=DA_PGCGROUP;
-}
+  /* common part of both <menus> and <titles> start. */
+  {
+    setvideo = 0;
+    setaudio = -1;
+    setsubpicture = -1;
+    subpmode = DA_PGCGROUP;
+  } /*pgcgroup_start*/
 
 static void titles_start()
-{
-    if( titles ) {
-        fprintf(stderr,"ERR:  Titles already defined\n");
-        parser_err=1;
-    } else if( istoc ) {
-        fprintf(stderr,"ERR:  Cannot have titles in a VMGM\n");
-        parser_err=1;
-    } else {
-        titles=pgcgroup_new(0);
-        curgroup=titles;
-        ismenuf=0;
+  /* called on a <titles> tag. */
+  {
+    if (titles)
+      {
+        fprintf(stderr, "ERR:  Titles already defined\n");
+        parser_err = 1;
+      }
+    else if (istoc)
+      {
+        fprintf(stderr, "ERR:  Cannot have titles in a VMGM\n");
+        parser_err = 1;
+      }
+    else
+      {
+        titles = pgcgroup_new(VTYPE_VTS);
+        curgroup = titles;
+        ismenuf = VTYPE_VTS;
         pgcgroup_start();
-    }
-}
+      } /*if*/
+  }
 
 static void menus_start()
-{
-    ismenuf=(istoc?2:1);
-    curgroup=pgcgroup_new(ismenuf);
+  /* called on a <menus> tag. */
+  {
+    ismenuf = (istoc ? VTYPE_VMGM : VTYPE_VTSM);
+    curgroup = pgcgroup_new(ismenuf);
     pgcgroup_start();
-    strcpy(menulang,"en");
-}
+    strcpy(menulang, "en"); /* default. */
+      /* fixme: would be nice to make this a preference setting, perhaps default to locale? */
+      /* fixme: no check for buffer overflow! */
+  }
 
-static void menus_lang(char *lang)
-{
-    strcpy(menulang,lang);
-}
+static void menus_lang(const char *lang)
+  {
+    strcpy(menulang, lang);
+      /* fixme: no check for buffer overflow! */
+  } /*menus_lang*/
 
 static void menus_end()
-{
-    menugroup_add_pgcgroup(mg,menulang,curgroup);
-    curgroup=0;
-}
+  {
+    menugroup_add_pgcgroup(mg, menulang, curgroup);
+    curgroup = 0;
+  } /*menus_end*/
 
 static void video_start()
-{
-    if( setvideo ) {
-        fprintf(stderr,"ERR:  Already defined video characteristics for this PGC group\n");
-        parser_err=1;
-    } else
-        setvideo=1;
-}
+  /* called on a <video> tag. */
+  {
+    if (setvideo)
+      {
+        fprintf(stderr, "ERR:  Already defined video characteristics for this PGC group\n");
+          /* only one video stream allowed */
+        parser_err = 1;
+      }
+    else
+        setvideo = 1;
+  }
 
-static void video_format(char *c)
-{
-    set_video_attr(VIDEO_FORMAT,c);
-}
+static void video_format(const char *c)
+  {
+    set_video_attr(VIDEO_FORMAT, c);
+  }
 
-static void video_aspect(char *c)
-{
-    set_video_attr(VIDEO_ASPECT,c);
-}
+static void video_aspect(const char *c)
+  {
+    set_video_attr(VIDEO_ASPECT, c);
+  }
 
-static void video_resolution(char *c)
-{
-    set_video_attr(VIDEO_RESOLUTION,c);
-}
+static void video_resolution(const char *c)
+  {
+    set_video_attr(VIDEO_RESOLUTION, c);
+  }
 
-static void video_widescreen(char *c)
-{
-    set_video_attr(VIDEO_WIDESCREEN,c);
-}
+static void video_widescreen(const char *c)
+  {
+    set_video_attr(VIDEO_WIDESCREEN, c);
+  }
 
-static void video_caption(char *c)
-{
-    set_video_attr(VIDEO_CAPTION,c);
-}
+static void video_caption(const char *c)
+  {
+    set_video_attr(VIDEO_CAPTION, c);
+  }
 
 static void audio_start()
-{
+  /* called on an <audio> tag. */
+  {
     setaudio++;
-    if( setaudio>=8 ) {
-        fprintf(stderr,"ERR:  Attempting to define too many audio streams for this PGC group\n");
-        parser_err=1;
-    }
-}
+    if (setaudio >= 8)
+      {
+        fprintf(stderr, "ERR:  Attempting to define too many audio streams for this PGC group\n");
+        parser_err = 1;
+      } /*if*/
+  }
 
-static void audio_format(char *c)
-{
-    set_audio_attr(AUDIO_FORMAT,c,setaudio);
-}
+static void audio_format(const char *c)
+  {
+    set_audio_attr(AUDIO_FORMAT, c, setaudio);
+  }
 
-static void audio_quant(char *c)
-{
-    set_audio_attr(AUDIO_QUANT,c,setaudio);
-}
+static void audio_quant(const char *c)
+  {
+    set_audio_attr(AUDIO_QUANT, c, setaudio);
+  }
 
-static void audio_dolby(char *c)
-{
-    set_audio_attr(AUDIO_DOLBY,c,setaudio);
-}
+static void audio_dolby(const char *c)
+  {
+    set_audio_attr(AUDIO_DOLBY, c, setaudio);
+  }
 
-static void audio_lang(char *c)
-{
-    set_audio_attr(AUDIO_LANG,c,setaudio);
-}
+static void audio_lang(const char *c)
+  {
+    set_audio_attr(AUDIO_LANG, c, setaudio);
+  }
 
-static void audio_samplerate(char *c)
-{
-    set_audio_attr(AUDIO_SAMPLERATE,c,setaudio);
-}
+static void audio_samplerate(const char *c)
+  {
+    set_audio_attr(AUDIO_SAMPLERATE, c, setaudio);
+  }
 
-static void audio_channels(char *c)
-{
+static void audio_content(const char *c)
+  {
+    set_audio_attr(AUDIO_CONTENT, c, setaudio);
+  } /*audio_content*/
+
+static void audio_channels(const char *c)
+  {
     char ch[4];
-
-    if( strlen(c) == 1 ) {
-        ch[0]=c[0];
-        ch[1]='c';
-        ch[2]='h';
-        ch[3]=0;
-        c=ch;
-    }
-    set_audio_attr(AUDIO_CHANNELS,ch,setaudio);
-}
+    if (strlen(c) == 1)
+      {
+        ch[0] = c[0];
+        ch[1] = 'c';
+        ch[2] = 'h';
+        ch[3] = 0;
+        c = ch;
+      } /*if*/
+    set_audio_attr(AUDIO_CHANNELS, ch, setaudio);
+  }
 
 static void subattr_group_start()
-{
-    if( subpmode!=DA_PGCGROUP ) {
-        fprintf(stderr,"ERR:  Define all the subpictures before defining PGCs\n");
-        parser_err=1;
+  /* called for a <subpicture> tag immediately within a pgcgroup (<menus> or <titles>). */
+ {
+    if (subpmode != DA_PGCGROUP)
+      {
+        fprintf(stderr, "ERR:  Define all the subpictures before defining PGCs\n");
+        parser_err = 1;
         return;
-    }
+      } /*if*/
     setsubpicture++;
-    if( setsubpicture>=32 ) {
-        fprintf(stderr,"ERR:  Attempting to define too many subpicture streams for this PGC group\n");
-        parser_err=1;
-    }
-}
+    if (setsubpicture >= 32)
+      {
+        fprintf(stderr, "ERR:  Attempting to define too many subpicture streams for this PGC group\n");
+        parser_err = 1;
+      } /*if*/
+  } /*subattr_group_start*/
 
 static void subattr_pgc_start()
-{
+  /* called for a <subpicture> tag immediately within a <pgc>. */
+  {
     setsubpicture++;
-    if( setsubpicture>=32 ) {
-        fprintf(stderr,"ERR:  Attempting to define too many subpicture streams for this PGC group\n");
-        parser_err=1;
-    }
-}
+    if (setsubpicture >= 32)
+      {
+        fprintf(stderr, "ERR:  Attempting to define too many subpicture streams for this PGC group\n");
+        parser_err = 1;
+      } /*if*/
+  } /*subattr_pgc_start*/
 
-static void subattr_lang(char *c)
-{
-    if( subpmode==DA_PGCGROUP )
-        set_subpic_attr(SPU_LANG,c,setsubpicture);
-    else {
-        fprintf(stderr,"ERR:  Cannot set subpicture language within a pgc; do it within titles or menus\n");
-        parser_err=1;
-    }
-}
+static void subattr_lang(const char *c)
+  {
+    if (subpmode == DA_PGCGROUP)
+        set_subpic_attr(SPU_LANG, c, setsubpicture);
+    else
+      {
+        fprintf
+          (
+            stderr,
+            "ERR:  Cannot set subpicture language within a pgc; do it within titles or menus\n"
+          );
+        parser_err = 1;
+      } /*if*/
+  } /*subattr_lang*/
+
+static void subattr_content(const char * c)
+  {
+    if (subpmode == DA_PGCGROUP)
+      {
+        set_subpic_attr(SPU_CONTENT, c, setsubpicture);
+      }
+    else
+      {
+        fprintf
+          (
+            stderr,
+            "ERR:  Cannot set subpicture content type within a pgc; do it within titles or menus\n"
+          );
+        parser_err = 1;
+      } /*if*/
+  } /*subattr_content*/
 
 static void stream_start()
-{
-    subpstreamid=-1;
-    subpstreammode=0;
-}
+  /* called on a <stream> tag. */
+  {
+    subpstreamid = -1;
+    subpstreammode = 0;
+  } /*stream_start*/
 
-static void substream_id(char *c)
-{
-    subpstreamid=atoi(c);
-    if( subpstreamid<0 || subpstreamid>=32 ) {
-        fprintf(stderr,"ERR:  Subpicture stream id must be 0-31: '%s'.\n",c);
-        parser_err=1;
-    }
-}
+static void substream_id(const char *c)
+  /* saves the id attribute for the current <stream> tag. */
+  {
+    subpstreamid = strtounsigned(c, "subpicture stream id");
+    if (subpstreamid < 0 || subpstreamid >= 32)
+      {
+        fprintf(stderr, "ERR:  Subpicture stream id must be 0-31: '%s'.\n", c);
+        parser_err = 1;
+      } /*if*/
+  } /*substream_id*/
 
-static void substream_mode(char *c)
-{
-    subpstreammode=strdup(c);
-}
+static void substream_mode(const char *c)
+  /* saves the mode attribute for the current <stream> tag. */
+  {
+    subpstreammode = strdup(c); /* won't leak, because I won't be called more than once */
+  }
 
 static void stream_end()
-{
-    if( subpstreamid==-1 || !subpstreammode ) {
-        fprintf(stderr,"ERR:  Must define the mode and id for the stream.\n");
-        parser_err=1;
-    } else if( subpmode==DA_PGCGROUP ) {
-        set_subpic_stream(setsubpicture,subpstreammode,subpstreamid);
-    } else
-        pgc_set_subpic_stream(curpgc,setsubpicture,subpstreammode,subpstreamid);
+  {
+    if (subpstreamid == -1 || !subpstreammode)
+      {
+        fprintf(stderr, "ERR:  Must define the mode and id for the stream.\n");
+        parser_err = 1;
+      }
+    else if (subpmode == DA_PGCGROUP)
+      {
+        set_subpic_stream(setsubpicture, subpstreammode, subpstreamid);
+      }
+    else
+        pgc_set_subpic_stream(curpgc, setsubpicture, subpstreammode, subpstreamid);
     free(subpstreammode);
-}
-
+  } /*stream_end*/
 
 static void pgc_start()
 {
-    curpgc=pgc_new();
-    hadchapter=0;
-    setsubpicture=-1;
-    subpmode=DA_PGC;
+    curpgc = pgc_new();
+    hadchapter = chapters_neither; /* reset for new menu/title */
+    setsubpicture = -1;
+    subpmode = DA_PGC;
 }
 
-static void pgc_entry(char *e)
+static void pgc_entry(const char *e)
 {
     // xml attributes can only be defined once, so entry="foo" entry="bar" won't work
     // instead, use parseentries...
-    parseentries(curpgc,e);
+    parseentries(curpgc, e);
 }
 
-static void pgc_palette(char *p)
+static void pgc_palette(const char *p)
 {
     readpalette(curpgc,p);
 }
 
-static void pgc_pause(char *c)
+static void pgc_pause(const char *c)
 {
     pgc_set_stilltime(curpgc,parse_pause(c));
 }
@@ -975,98 +1136,119 @@ static void post_end()
 
 static void vob_start()
 {
-    curvob=source_new();
-    pauselen=0;
-    vobbasic=0;
-    cell_endtime=0;
+    curvob = source_new();
+    pauselen = 0;
+    vobbasic = vob_has_neither; /* to begin with */
+  /* but note hadchapter keeps its value from previous VOB in this PGC
+    if no chapters attribute or cells are seen */
+    cell_endtime = 0;
 }
 
-static void vob_file(char *f)
+static void vob_file(const char *f)
 {
-    f=utf8tolocal(f);
-    source_set_filename(curvob,f);
-    free(f);
+    f = utf8tolocal(f);
+    source_set_filename(curvob, f);
 }
 
-static void vob_chapters(char *c)
-{
-    vobbasic=1;
-    hadchapter=2;
-    chapters=strdup(c);
-}
+static void vob_chapters(const char *c)
+  {
+    vobbasic = vob_has_chapters_pause;
+    hadchapter = chapters_chapters;
+    chapters = strdup(c); /* fixme: leaks! */
+  }
 
-static void vob_pause(char *c)
-{
-    vobbasic=1;
-    pauselen=parse_pause(c);
-}
+static void vob_pause(const char *c)
+  {
+    vobbasic = vob_has_chapters_pause;
+    pauselen = parse_pause(c);
+  }
 
 static void vob_end()
-{
-    if( vobbasic!=2 ) {
-        if( hadchapter==2 ) {
-            parsechapters(chapters,curvob,pauselen);
-            hadchapter=1;
-        } else
-            source_add_cell(curvob,0,-1,!hadchapter,pauselen,0);
-    }
-    pgc_add_source(curpgc,curvob);
-    curvob=0;
-}
+  {
+    if (vobbasic != vob_has_cells) /* = vob_has_chapters_pause or vob_has_neither */
+      {
+        if (hadchapter == chapters_chapters) /* vob has chapters attribute */
+          {
+          /* fixme: hadchapter could still be set to this value from previous vob,
+            in which case I wrongly apply same chapter definitions to this vob! */
+            parsechapters(chapters, curvob, pauselen);
+            hadchapter = chapters_cells;
+          }
+        else /* hadchapter = chapters_neither or chapters_cells */
+          /* I think this should only ever be chapters_neither */
+            source_add_cell
+              (
+                /*source =*/ curvob,
+                /*starttime =*/ 0,
+                /*endtime =*/ -1,
+                /*chap =*/ hadchapter == chapters_neither ? CELL_CHAPTER_PROGRAM : CELL_NEITHER,
+                /*pause =*/ pauselen,
+                /*cmd =*/ 0
+              );
+              /* default to single chapter for entire source file */
+      } /*if*/
+    pgc_add_source(curpgc, curvob);
+    curvob = 0;
+  } /*vob_end*/
 
 static void cell_start()
-{
-    parser_acceptbody=1;
-    assert(vobbasic!=1);
-    vobbasic=2;
-    cell_starttime=cell_endtime;
-    cell_endtime=-1;
-    cell_chapter=0;
-    pauselen=0;
-    hadchapter=1;
-}
+  {
+    parser_acceptbody = 1; /* collect cell commands */
+    assert(vobbasic != vob_has_chapters_pause);
+      /* no "chapters" or "pause" attribute on containing <vob> allowed */
+    vobbasic = vob_has_cells;
+    cell_starttime = cell_endtime; /* new cell starts by default where previous one ends */
+    cell_endtime = -1;
+    cell_chapter = CELL_NEITHER; /* to begin with */
+    pauselen = 0;
+    hadchapter = chapters_cells;
+  }
 
-static void cell_parsestart(char *f)
-{
-    cell_starttime=parsechapter(f);
-}
+static void cell_parsestart(const char *f)
+  {
+    cell_starttime = parsechapter(f);
+  }
 
-static void cell_parseend(char *f)
-{
-    cell_endtime=parsechapter(f);
-}
+static void cell_parseend(const char *f)
+  {
+    cell_endtime = parsechapter(f);
+  }
 
-static void cell_parsechapter(char *f)
-{
-    int i=xml_ison(f);
-    if(i==-1) {
-        fprintf(stderr,"ERR:  Unknown chapter cmd '%s'\n",f);
+static void cell_parsechapter(const char *f)
+  {
+    const int i = xml_ison(f);
+    if (i == -1)
+      {
+        fprintf(stderr, "ERR:  Unknown chapter cmd '%s'\n", f);
         exit(1);
-    } else if (i)
-        cell_chapter=1;
-}
+      }
+    else if (i)
+        cell_chapter = CELL_CHAPTER_PROGRAM;
+  }
 
-static void cell_parseprogram(char *f)
-{
-    int i=xml_ison(f);
-    if(i==-1) {
-        fprintf(stderr,"ERR:  Unknown program cmd '%s'\n",f);
+static void cell_parseprogram(const char *f)
+  {
+    const int i = xml_ison(f);
+    if (i == -1)
+      {
+        fprintf(stderr, "ERR:  Unknown program cmd '%s'\n", f);
         exit(1);
-    } else if (i && cell_chapter!=1 )
-        cell_chapter=2;
-}
+      }
+    else if (i && cell_chapter != CELL_CHAPTER_PROGRAM)
+        cell_chapter = CELL_PROGRAM;
+  }
 
-static void cell_pauselen(char *f)
-{
+static void cell_pauselen(const char *f)
+  {
     pauselen=parse_pause(f);
-}
+  }
 
 static void cell_end()
-{
-    assert( cell_starttime>=0 );
-    source_add_cell(curvob,cell_starttime,cell_endtime,cell_chapter,pauselen,parser_body);
-    pauselen=0;
-}
+  {
+    assert (cell_starttime >= 0);
+    source_add_cell(curvob, cell_starttime, cell_endtime, cell_chapter, pauselen, parser_body);
+    pauselen = 0;
+  }
 
 static void button_start()
 {
@@ -1074,7 +1256,7 @@ static void button_start()
     buttonname=0;
 }
 
-static void button_name(char *f)
+static void button_name(const char *f)
 {
     buttonname=strdup(f);
 }
@@ -1082,7 +1264,7 @@ static void button_name(char *f)
 static void button_end()
 {
     pgc_add_button(curpgc,buttonname,parser_body);
-    if(buttonname) free(buttonname);
+    if(buttonname) free((char *)buttonname);
 }
 
 static struct elemdesc elems[]={
@@ -1137,8 +1319,10 @@ static struct elemattr attrs[]={
     {"audio","lang",audio_lang},
     {"audio","channels",audio_channels},
     {"audio","samplerate",audio_samplerate},
+    {"audio","content",audio_content},
 
     {"subpicture","lang",subattr_lang},
+    {"subpicture","content",subattr_content},
     {"stream","mode",substream_mode},
     {"stream","id",substream_id},
 
@@ -1148,7 +1332,7 @@ static struct elemattr attrs[]={
     {0,0,0}
 };
 
-static int readdvdauthorxml(const char *xmlfile,char *fb)
+static int readdvdauthorxml(const char *xmlfile,const char *fb)
 {
     fbase=fb;
     return readxml(xmlfile,elems,attrs);
