@@ -1,3 +1,7 @@
+/*
+    Utility to extract audio/video streams and dump information about
+    packetes in an MPEG stream.
+*/
 /*
  * Copyright (C) 2002 Scott Smith (trckjunky@users.sourceforge.net)
  *
@@ -282,7 +286,7 @@ int main(int argc,char **argv)
     while( -1 != (oc=getopt(argc,argv,"ha:v:o:msd:u")) ) {
         switch(oc) {
         case 'd':
-            audiodrop=atoi(optarg);
+            audiodrop=strtounsigned(optarg, "audio drop count");
             break;
 
         case 'a':
@@ -291,7 +295,7 @@ int main(int argc,char **argv)
                 fprintf(stderr,"can only output one stream to stdout at a time\n; use -o to output more than\none stream\n");
                 exit(1);
             }
-            outputstream=((oc=='a')?0xc0:0xe0)+atoi(optarg);
+            outputstream=((oc=='a')?0xc0:0xe0)+strtounsigned(optarg, "stream id");
             break;
 
         case 'm':
@@ -458,7 +462,7 @@ int main(int argc,char **argv)
             int64_t fulltime;
             forceread(buf,8,stdin);
             if((buf[0] & 0xC0) == 0x40) {
-	        forceread(buf+8,2,stdin);
+            forceread(buf+8,2,stdin);
                 scrhi=(buf[0]&0x20)>>5;
                 scr=((buf[0]&0x18)<<27)|
                     ((buf[0]&3)<<28)|
@@ -476,11 +480,11 @@ int main(int argc,char **argv)
                 fulltime*=300;
                 fulltime+=scrext;
                 mpeg2 = 1;
-	    } else if((buf[0] & 0xF0) == 0x20) {
+        } else if((buf[0] & 0xF0) == 0x20) {
                 mpeg2 = 0;
-		fulltime=readpts(buf);
+        fulltime=readpts(buf);
                 fulltime*=300;
-	    } else {
+        } else {
                 if( outputenglish )
                     printf("WARN: unknown pack header version\n");
                 fulltime=0;
@@ -583,40 +587,71 @@ int main(int argc,char **argv)
             readlen=forceread(buf,(extra>sizeof(buf))?sizeof(buf):extra,stdin);
             extra-=readlen;
             if( outputenglish ) {
-                if( ntohl(hdr)==0x1bd ) { // private stream 1
-                    int sid=buf[3+buf[2]];
-                    switch( sid&0xe0 ) {
-                    case 0x20: printf(", subpicture %d",sid&0x1f); break;
-                    case 0x80: printf(", audio %d",sid&0x1f); break;
-                    case 0xa0: printf(", lpcm %d",sid&0x1f); break;
-                    default: printf(", substream id 0x%02x",sid); break;
-                    }
-                }
+                if (ntohl(hdr) == 0x1bd) // private stream 1
+                  {
+                    const int sid = buf[3 + buf[2]]; /* substream ID is first byte after header */
+                    switch (sid & 0xf8)
+                      {
+                    case 0x20:
+                    case 0x28:
+                    case 0x30:
+                    case 0x38:
+                        printf(", subpicture %d", sid & 0x1f);
+                    break;
+                    case 0x80:
+                        printf(", AC3 audio %d", sid & 7);
+                    break;
+                    case 0x88:
+                        printf(", DTS audio %d", sid & 7);
+                    case 0xa0:
+                        printf(", LPCM audio %d", sid & 7);
+                    break;
+                    default:
+                        printf(", substream id 0x%02x", sid);
+                    break;
+                      } /*switch*/
+                  }
+                else if (ntohl(hdr) == 0x1bf) // private stream 2
+                  {
+                    const int sid = buf[0];
+                    switch (sid)
+                      {
+                    case 0:
+                        printf(", PCI");
+                    break;
+                    case 1:
+                        printf(", DSI");
+                    break;
+                    default:
+                        printf(", substream id 0x%02x", sid);
+                    break;
+                      } /*switch*/
+                  } /*if*/
                 printf("; length=%d",extra+readlen);
                 if( ext ) {
                     int eptr=3;
-		    int hdr=0, has_pts, has_dts, has_std=0, std=0, std_scale=0;
-		    
-		    if((buf[0] & 0xC0) == 0x80) {
-		        mpeg2 = 1;
-			hdr = buf[2]+3;
-			eptr = 3;
-			has_pts = buf[1] & 128;
-			has_dts = buf[1] & 64;
+            int hdr=0, has_pts, has_dts, has_std=0, std=0, std_scale=0;
+            
+            if((buf[0] & 0xC0) == 0x80) {
+                mpeg2 = 1;
+            hdr = buf[2]+3;
+            eptr = 3;
+            has_pts = buf[1] & 128;
+            has_dts = buf[1] & 64;
                     }
                     else {
-		        mpeg2 = 0;
-			while((buf[hdr] == 0xff) && (hdr<sizeof(buf)))
-			    hdr++;
+                mpeg2 = 0;
+            while((buf[hdr] == 0xff) && (hdr<sizeof(buf)))
+                hdr++;
                         if((buf[hdr] & 0xC0) == 0x40) {
-			    has_std = 1;
-			    std_scale = ((buf[hdr]&32)?1024:128);
-			    std = ((buf[hdr]&31)*256+buf[hdr+1])*std_scale;
-			    hdr+=2;
+                has_std = 1;
+                std_scale = ((buf[hdr]&32)?1024:128);
+                std = ((buf[hdr]&31)*256+buf[hdr+1])*std_scale;
+                hdr+=2;
                         } else has_std = 0;
                         eptr = hdr;
-			has_pts = (buf[hdr] & 0xE0) == 0x20;
-			has_dts = (buf[hdr] & 0xF0) == 0x30;
+            has_pts = (buf[hdr] & 0xE0) == 0x20;
+            has_dts = (buf[hdr] & 0xF0) == 0x30;
                     }
 
                     printf("; hdr=%d",hdr);
@@ -634,7 +669,7 @@ int main(int argc,char **argv)
                         eptr+=5;
                         printf("; dts %" PRId64 ".%03" PRId64 " sec",dts/PTSTIME,(dts%PTSTIME)/(PTSTIME/1000));
                     }
-		    if(mpeg2) {
+            if(mpeg2) {
                         if( buf[1] & 32 ) {
                             printf("; escr");
                             eptr+=6;
@@ -677,10 +712,10 @@ int main(int argc,char **argv)
                                 eptr+=2;
                             }
                         }
-		    } else {
+            } else {
                         if(has_std)
                             printf("; pstd=%d (scale=%d)",std, std_scale);
-		    }
+            }
                 }
                 printf("\n");
             }
