@@ -1,3 +1,6 @@
+/*
+    Generation of subpicture encoding
+*/
 /*
  * Copyright (C) 2002, 2003 Jan Panteltje <panteltje@yahoo.com>
  * With many changes by Scott Smith (trckjunky@users.sourceforge.net)
@@ -25,7 +28,7 @@
 #include <assert.h>
 
 #include "subgen.h"
-
+#include "common.h"
 
 static int remainbit,subo;
 
@@ -133,16 +136,16 @@ int svcd_encode(stinfo *s)
     subo = 2;
     if (s->sd != -1)
     {
-	store_2(0x2e00);
+    store_2(0x2e00);
         store_4(s->sd);
     }
     else
     {
-	store_2(0x2600);
+    store_2(0x2600);
     } 
  
     if (debug > 2)
-	fprintf(stderr,\
+    fprintf(stderr,\
                 "sd: %d   xd: %d  yd: %d  x0: %d  y0: %d\n", s->sd, s->xd, s->yd, s->x0, s->y0); 
 
     store_2(s->x0);
@@ -151,10 +154,10 @@ int svcd_encode(stinfo *s)
     store_2(s->yd);
     for(c = 0;c<4;c++)
     {
-	store_1(calcY(&epal[c]));
-	store_1(calcCr(&epal[c]));
-	store_1(calcCb(&epal[c]));
-	store_1(epal[c].t);
+    store_1(calcY(&epal[c]));
+    store_1(calcCr(&epal[c]));
+    store_1(calcCb(&epal[c]));
+    store_1(epal[c].t);
     } 
  
     store_1(0); //?????
@@ -165,7 +168,7 @@ int svcd_encode(stinfo *s)
  odd_row: 
     for(; y<s->yd; y += 2)
     {
-	for(x = 0; x<s->xd;x++)
+    for(x = 0; x<s->xd;x++)
         {
             if ((c = s->fimg[y*s->xd+x]) != 0) store_2bit(c);
             else
@@ -186,24 +189,24 @@ int svcd_encode(stinfo *s)
  
     if (!(y&1))
     {
-	if (!(subo&1))
+    if (!(subo&1))
         {
             if (debug>3)
                 fprintf(stderr,\
-			"padded betweed fields with 1 byte to %d\n",subo%4);
+            "padded betweed fields with 1 byte to %d\n",subo%4);
             store_1( 0 );
         }
-	y = 1;
-	sub[l2o] = (subo - l2o - 2) >> 8;
-	sub[l2o+1] = (subo - l2o - 2);
-	goto odd_row;
+    y = 1;
+    sub[l2o] = (subo - l2o - 2) >> 8;
+    sub[l2o+1] = (subo - l2o - 2);
+    goto odd_row;
     } 
  
     store_1( 0 );// no additional commands
     c = 0;
     while (subo&3)
     {
-	store_1( 0 ); c++;
+    store_1( 0 ); c++;
     }
     if (debug>3) fprintf(stderr,"padded with %d byte\n",c);
 
@@ -228,7 +231,7 @@ int cvd_encode(stinfo *s)
     for(y = 0; y < s->yd; y += 2)
     {
     odd_row_cvd: 
-	for(x = 0; x < s->xd;)
+    for(x = 0; x < s->xd;)
         {
             d = s->fimg[y * s->xd + x];
             c = 1;
@@ -240,7 +243,7 @@ int cvd_encode(stinfo *s)
                 store_align();
                 continue;
             }
-	   
+       
             while(c > 3)
             {
                 store_nibble(12 + d);
@@ -253,9 +256,9 @@ int cvd_encode(stinfo *s)
  
     if(!(y & 1))
     {
-	y = 1;
-	ofs1 = subo;
-	goto odd_row_cvd;
+    y = 1;
+    ofs1 = subo;
+    goto odd_row_cvd;
     }  
  
     sub[2] = subo >> 8;
@@ -271,37 +274,37 @@ int cvd_encode(stinfo *s)
     for(c = 0; c < 4; c++)
     {
 //#define nco if (subo<65536) sub[subo++]
-	store_1( 0x24 + c );
-	if(debug > 3)
+    store_1( 0x24 + c );
+    if(debug > 3)
         {
             fprintf(stderr, "c=%d R=%.2f G=%.2f B=%.2f\n",\
                     c, (double)epal[c].r, (double)epal[c].g, (double)epal[c].b);
         }
 
-	store_1( calcY(&epal[c]) );
-	store_1( calcCr(&epal[c]) );
-	store_1( calcCb(&epal[c]) );
+    store_1( calcY(&epal[c]) );
+    store_1( calcCr(&epal[c]) );
+    store_1( calcCb(&epal[c]) );
 
     } /* end for pallette 0-3 */
  
 /* sethighlight  pallette  */
     for(c = 0; c < 4; c++)
     {
-	store_1( 0x2c + c );
-	if(debug > 3)
+    store_1( 0x2c + c );
+    if(debug > 3)
         {
             fprintf(stderr, "c=%d R=%.2f G=%.2f B=%.2f\n",\
                     c, (double)epal[c].r, (double)epal[c].g, (double)epal[c].b);
         }
 
-	store_1( calcY(&epal[c]) );
-	store_1( calcCr(&epal[c]) );
-	store_1( calcCb(&epal[c]) );
+    store_1( calcY(&epal[c]) );
+    store_1( calcCr(&epal[c]) );
+    store_1( calcCb(&epal[c]) );
     } /* end for pallette 4-7 */
 
     if(debug > 3)
     {
-	fprintf(stderr,\
+    fprintf(stderr,\
                 "epal[0].t=%d epal[1].t=%d epal[2].t=%d epal[3].t=%d\n",\
                 epal[0].t, epal[1].t, epal[2].t, epal[3].t);
     }
@@ -327,7 +330,7 @@ int cvd_encode(stinfo *s)
 
     if(debug > 3)
     {
-	fprintf(stderr, "EPALS nco0(2 3h, 2l)=%02x nco1(2 1h,0l)=%02x\n",\
+    fprintf(stderr, "EPALS nco0(2 3h, 2l)=%02x nco1(2 1h,0l)=%02x\n",\
                 sub[subo - 1], sub[subo - 2]);
     }
 
@@ -388,31 +391,31 @@ static void do_rle(int count, int color)
 
     /* a now ranges from 0x4 up, because count is at least 1 */
 
-    if(count >= 64)		// 64 - 255
+    if(count >= 64)     // 64 - 255
     {
-	/* 64-255, 16 bits, 0 0 0 0  0 0 n n  n n n n  n n c c */
-	store_nibble(0);
-	store_nibble( (a & 0xf00) >> 8);
-	store_nibble( (a & 0xf0) >> 4);
-	store_nibble( a & 0xf);
+    /* 64-255, 16 bits, 0 0 0 0  0 0 n n  n n n n  n n c c */
+    store_nibble(0);
+    store_nibble( (a & 0xf00) >> 8);
+    store_nibble( (a & 0xf0) >> 4);
+    store_nibble( a & 0xf);
     }
-    else if(count >= 16)	// 16 - 63
+    else if(count >= 16)    // 16 - 63
     {
-	/* 16 - 63, 12 bits, 0 0 0 0  n n n n  n n c c */
-	store_nibble( 0);
-	store_nibble( (a & 0xf0) >> 4);
-	store_nibble( a & 0xf);
+    /* 16 - 63, 12 bits, 0 0 0 0  n n n n  n n c c */
+    store_nibble( 0);
+    store_nibble( (a & 0xf0) >> 4);
+    store_nibble( a & 0xf);
     }
-    else if(count >= 4)		// 4 - 15
+    else if(count >= 4)     // 4 - 15
     {
-	/* 4-15, 8 bits, 0 0 n n  n n c c */
-	store_nibble( (a & 0xf0) >> 4);
-	store_nibble( a & 0xf);
+    /* 4-15, 8 bits, 0 0 n n  n n c c */
+    store_nibble( (a & 0xf0) >> 4);
+    store_nibble( a & 0xf);
     }
-    else			// 1 - 3
+    else            // 1 - 3
     {
-	/* 1-3, 4 bits, n n c c */
-	store_nibble( a & 0xf );
+    /* 1-3, 4 bits, n n c c */
+    store_nibble( a & 0xf );
     }
 } /* end function do_rle */
 
@@ -446,7 +449,7 @@ static void dvd_encode_row(int y,int xd,unsigned char *icptr)
       encoding a count of zero using the 16-bit format,
       indicates the same pixel value until the end of the line. 
     */
-	
+    
     if( xd != new_pos )
     {
         int count=xd-new_pos;
@@ -516,11 +519,11 @@ int dvd_encode(stinfo *s)
 
     /* copy image data to sub */
     offset0=subo;
-    for( y=0; y<s->yd; y+=2 )
+    for( y=0; y<s->yd; y+=2 ) /* top field */
         dvd_encode_row(y,s->xd,icptr);
 
     offset1=subo;
-    for( y=1; y<s->yd; y+=2 )
+    for( y=1; y<s->yd; y+=2 ) /* bottom field */
         dvd_encode_row(y,s->xd,icptr);
 
     /* start first command block */
@@ -539,8 +542,8 @@ int dvd_encode(stinfo *s)
    If this is the last SP_DCSQ, it points to itself.
 
    Converting frames and time to SP_DCSQ_STM values
-   The direct method of converting time to delay values is to multiply time in seconds by 90000/1024 and
-   truncate the value.
+   The direct method of converting time to delay values is to multiply time in seconds
+   by 90000/1024 and truncate the value.
    Rounding up will cause the display to occur one frame late.
 */
 
@@ -557,43 +560,40 @@ int dvd_encode(stinfo *s)
 
     
     if( s->forced )
-        store_1(0); /* command 0, forced start display, 1 byte. */
+        store_1(SPU_FSTA_DSP); /* command 0, forced start display, 1 byte. */
     else
-        store_1(1); /* command 1, start display, 1 byte */
+        store_1(SPU_STA_DSP); /* command 1, start display, 1 byte */
 
     /* selected palettes for pixel value */
 
     /* command 3, palette, 3 bytes */
-        store_1(3);
+    store_1(SPU_SET_COLOR);
     store_nibble(findmasterpal(s,&s->pal[3]));
     store_nibble(findmasterpal(s,&s->pal[2]));
     store_nibble(findmasterpal(s,&s->pal[1]));
     store_nibble(findmasterpal(s,&s->pal[0]));
 
     /* command 4, alpha blend, contrast / transparency t_palette, 3 bytes  0, 15, 15, 15 */
-
-    store_1(4);
+    store_1(SPU_SET_CONTR);
     store_nibble(s->pal[3].t>>4);
     store_nibble(s->pal[2].t>>4);
     store_nibble(s->pal[1].t>>4);
     store_nibble(s->pal[0].t>>4);
 
     /* command 5, display area, 7 bytes from: startx, xsize, starty, ysize */
-
-    store_1(5);
+    store_1(SPU_SET_DAREA);
     store_trinibble(xstart);
     store_trinibble(xstart+xsize-1);
     store_trinibble(ystart);
     store_trinibble(ystart+ysize-1);
 
     /* command 6, image offsets, 5 bytes */
-    store_1(6);
-
+    store_1(SPU_SET_DSPXA);
     store_2(offset0);
     store_2(offset1);
 
     /* command 0xff, end command block, 1 byte, */
-    store_1(0xff);
+    store_1(SPU_CMD_END);
 
     /* end first command block */
 
@@ -614,10 +614,12 @@ int dvd_encode(stinfo *s)
         /* delay to wait before executing next comand */
         duration = (s->sd+512)/1024;
         while( duration >= 65536 ) {
+          /* duration too long for one command block, generate additional command blocks
+            that do nothing but delay */
             store_2(65535);
             duration-=65535;
             store_2(next_command_ptr+5);
-            store_1(0xFF);
+            store_1(SPU_CMD_END);
             next_command_ptr = subo;
         }
             
@@ -627,10 +629,10 @@ int dvd_encode(stinfo *s)
         store_2(next_command_ptr);
 
         /* stop command (executed after above delay) */
-        store_1(0x02);
+        store_1(SPU_STP_DSP);
 
         /* end command block command */
-        store_1(0xff);
+        store_1(SPU_CMD_END);
     } else {
         a=next_command_ptr-2;
         sub[next_command_ptr+0] = a >> 8;
@@ -643,8 +645,8 @@ int dvd_encode(stinfo *s)
     /* make size even if odd */
     if(subo & 1)
     {
-	/* only if odd length, to make it even */
-	store_1(0xff);
+    /* only if odd length, to make it even */
+    store_1(SPU_CMD_END);
     }
 
     /* set subtitle packet size */
