@@ -73,7 +73,7 @@ int verbose=0;
 float osd_font_scale_factor = 6.0;
 float subtitle_font_radius = 0.0;     /*2.0*/
 float subtitle_font_thickness = 3.0;  /*2.0*/
-int subtitle_autoscale = 0; /* 0=no autoscale 1=video height 2=video width 3=diagonal */
+int subtitle_autoscale = AUTOSCALE_NONE;
 int sub_bg_color=8; /* subtitles background color */
 int sub_bg_alpha=0;
 int sub_justify=1;
@@ -87,8 +87,8 @@ char* sub_cp="ISO8859-1";    /* sub_cp (char) contains "from character-set" for
                           /* "to character-set" is set to UTF-8 (optional user parameter, NULL for non-applicable)*/
 float text_font_scale_factor = 28.0; /* font size in font units */
 int text_forceit = 0;     /* Forcing of the subtitles */
-int h_sub_alignment = 1;  /* Horizontal alignmeent 0=center, 1=left, 2=right, 4=subtitle default */
-int sub_alignment=2;      /* Vertical alignment 0=top, 1=center, 2=bottom */
+int h_sub_alignment = H_SUB_ALIGNMENT_LEFT;  /* Horizontal alignment 0=center, 1=left, 2=right, 4=subtitle default */
+int v_sub_alignment = V_SUB_ALIGNMENT_BOTTOM;      /* Vertical alignment 0=top, 1=center, 2=bottom */
 int sub_left_margin=60;   /* Size of left horizontal non-display area in pixel units */
 int sub_right_margin=60;  /* Size of right horizontal non-display area in pixel units */
 int sub_bottom_margin=30; /* Size of bottom horizontal non-display area in pixel units */
@@ -102,203 +102,232 @@ float movie_fps=25.0;
 int movie_width=720;
 int movie_height=574;
 sub_data *textsub_subdata;
-subtitle *textsub_subs;
 subtitle *vo_sub;
-unsigned char *image_buffer;
-char *font_name;
+unsigned char *textsub_image_buffer;
+char *textsub_font_name;
 int current_sub;
 font_desc_t* vo_font;
-char *filename=NULL;
 int sub_max_chars;
 int sub_max_lines;
 int sub_max_font_height;
 int sub_max_bottom_font_height;
-int sub_last;
-int sub_num_of_subtitles;
-char *img_name;
 
-sub_data * textsub_init(char *textsub_filename, float textsub_movie_fps, float textsub_movie_width, float textsub_movie_height)
-{
-  vo_sub=NULL;
-  font_name=NULL;
-  current_sub=-1;
-  vo_font=NULL;
-  sub_last=1;
-  sub_max_chars=0;
-  sub_max_lines=0;
-  sub_max_font_height=0;
-  sub_max_bottom_font_height=0;
-  sub_num_of_subtitles=0;
-  movie_fps=textsub_movie_fps;
-  movie_width=textsub_movie_width;
-  movie_height=textsub_movie_height;
-#ifdef HAVE_FREETYPE
-  if (!vo_font)
-	init_freetype();
-#endif
-  vo_init_osd();
-#ifdef ICONV
-  if (sub_cp)
-    if (!strcmp(sub_cp,""))
-      sub_cp=NULL;
-#endif
-  if (dvdsub_lang)
-    if (!strcmp(dvdsub_lang,""))
-      dvdsub_lang=NULL;
-  image_buffer=malloc(sizeof(uint8_t)*3*textsub_movie_height*textsub_movie_width*3);
-  memset(image_buffer,128,sizeof(uint8_t)*3*textsub_movie_height*textsub_movie_width*3);
-  if ( image_buffer==NULL)
+static int sub_last;
+static int sub_num_of_subtitles;
+
+sub_data * textsub_init
+  (
+    const char *textsub_filename,
+    float textsub_movie_fps,
+    float textsub_movie_width,
+    float textsub_movie_height
+  )
+  /* loads subtitles from textsub_filename and sets up structures for rendering
+    the text. */
   {
-    fprintf(stderr,"ERR: Failed to allocate memory\n");
-	exit(1);
-  }
-  textsub_subdata=sub_read_file(textsub_filename,textsub_movie_fps);
-  vo_update_osd(textsub_movie_width,textsub_movie_height);
-  vo_osd_changed(OSDTYPE_SUBTITLE);
-  if (textsub_subdata!=NULL)
-    textsub_subs=textsub_subdata->subtitles;
-  return(textsub_subdata);
-}
+    const size_t image_buffer_size =
+        sizeof(uint8_t) * 3 * textsub_movie_height * textsub_movie_width * 3;
+    vo_sub = NULL;
+    textsub_font_name = NULL; /* never set to any other value! */
+    current_sub = -1;
+    vo_font = NULL;
+    sub_last = 1;
+    sub_max_chars = 0;
+    sub_max_lines = 0;
+    sub_max_font_height = 0;
+    sub_max_bottom_font_height = 0;
+    sub_num_of_subtitles = 0;
+    movie_fps = textsub_movie_fps;
+    movie_width = textsub_movie_width;
+    movie_height = textsub_movie_height;
+#ifdef HAVE_FREETYPE
+    if (!vo_font)
+        init_freetype();
+#endif
+    vo_init_osd();
+#ifdef ICONV
+    if (sub_cp)
+        if (!strcmp(sub_cp, ""))
+            sub_cp = NULL;
+#endif
+    if (dvdsub_lang)
+        if (!strcmp(dvdsub_lang, ""))
+            dvdsub_lang = NULL;
+    textsub_image_buffer = malloc(image_buffer_size);
+      /* fixme: not freed from previous call! */
+    if (textsub_image_buffer == NULL)
+     {
+        fprintf(stderr, "ERR: Failed to allocate memory\n");
+        exit(1);
+      } /*if*/
+    memset(textsub_image_buffer, 128, image_buffer_size);
+    textsub_subdata = sub_read_file(textsub_filename, textsub_movie_fps);
+      /* fixme: sub_free never called! */
+    vo_update_osd(textsub_movie_width, textsub_movie_height);
+    vo_osd_changed(OSDTYPE_SUBTITLE);
+    return textsub_subdata;
+  } /*textsub_init*/
 
 void textsub_dump_file()
-{
-  list_sub_file(textsub_subdata);
-}
-
-textsub_subtitle_type *textsub_find_sub(unsigned long text_sub_pts)
-{
-  textsub_subtitle_type *tsub;
-
-  tsub=(textsub_subtitle_type*)malloc(sizeof(textsub_subtitle_type));
-  tsub->valid=0;
-  tsub->start=-1;
-  find_sub(textsub_subdata,text_sub_pts);
-  if ( (vo_sub)&& (current_sub!=sub_last))
   {
-    if ( h_sub_alignment!=SUB_ALIGNMENT_DEFAULT)
-	{
-	  vo_sub->alignment=h_sub_alignment;
-	}
-	vo_sub->text_forced=text_forceit;   // Not sure where this should go... PMD
-	sub_num_of_subtitles++;
-	sub_last=current_sub;
-	tsub->start=vo_sub->start;
-	tsub->end=vo_sub->end;
-	tsub->valid=1;
-  }
-  return (tsub);
-}
+    list_sub_file(textsub_subdata);
+  } /*textsub_dump_file*/
+
+textsub_subtitle_type textsub_find_sub(unsigned long text_sub_pts)
+  {
+    textsub_subtitle_type result;
+    result.valid = 0;
+    result.start = -1;
+    find_sub(textsub_subdata, text_sub_pts);
+    if (vo_sub && current_sub != sub_last)
+      {
+        if (h_sub_alignment != H_SUB_ALIGNMENT_DEFAULT)
+          {
+            vo_sub->alignment = h_sub_alignment;
+          } /*if*/
+        vo_sub->text_forced = text_forceit;   // Not sure where this should go... PMD
+        sub_num_of_subtitles++;
+        sub_last = current_sub;
+        result.start = vo_sub->start;
+        result.end = vo_sub->end;
+        result.valid = 1;
+      } /*if*/
+    return result;
+  } /*textsub_find_sub*/
 
 /* extern char *draw_image(int p_w, int p_h, unsigned char* p_planes,unsigned int p_stride); */
 
-void textsub_render(subtitle* sub)
-{
-  vo_sub=sub;
-  vo_osd_changed(OSDTYPE_SUBTITLE);
-  memset(image_buffer,128,sizeof(uint8_t)*3*movie_height*movie_width*3);
-  vo_update_osd(movie_width,movie_height);
-/*  draw_image(movie_width,movie_height,image_buffer,movie_width*3); */
-}
+void textsub_render(subtitle * sub)
+  {
+    vo_sub = sub;
+    vo_osd_changed(OSDTYPE_SUBTITLE);
+    memset(textsub_image_buffer, 128, sizeof(uint8_t) * 3 * movie_height * movie_width * 3);
+    vo_update_osd(movie_width, movie_height);
+/*  draw_image(movie_width, movie_height, textsub_image_buffer, movie_width * 3); */
+  } /*textsub_render*/
 
 void textsub_statistics()
-{
-  fprintf(stderr,"\nStatistics:\n");
-  fprintf(stderr,"- Processed %d subtitles.\n",sub_num_of_subtitles);
-  fprintf(stderr,"- The longest display line had %d characters.\n",sub_max_chars-1);
-  fprintf(stderr,"- The maximum number of displayed lines was %d.\n",sub_max_lines);
-  fprintf(stderr,"- The normal display height of the font %s was %d.\n",sub_font,sub_max_font_height);
-  fprintf(stderr,"- The bottom display height of the font %s was %d.\n",sub_font,sub_max_bottom_font_height);
-  fprintf(stderr,"- The biggest subtitle box had %d bytes.\n",max_sub_size);
-}
+  {
+    fprintf(stderr, "\nStatistics:\n");
+    fprintf(stderr, "- Processed %d subtitles.\n", sub_num_of_subtitles);
+    fprintf(stderr, "- The longest display line had %d characters.\n", sub_max_chars - 1);
+    fprintf(stderr, "- The maximum number of displayed lines was %d.\n", sub_max_lines);
+    fprintf(stderr, "- The normal display height of the font %s was %d.\n", sub_font, sub_max_font_height);
+    fprintf(stderr, "- The bottom display height of the font %s was %d.\n", sub_font, sub_max_bottom_font_height);
+    fprintf(stderr, "- The biggest subtitle box had %d bytes.\n", max_sub_size);
+  } /*textsub_statistics*/
 
 void textsub_finish()
-{
-  free(image_buffer);
+  {
+    vo_finish_osd();
+    free(textsub_image_buffer);
 #ifdef HAVE_FREETYPE
-  if (vo_font) free_font_desc(vo_font);
+    free_font_desc(vo_font);
     vo_font = NULL;
-  done_freetype();
+    done_freetype();
 #endif
-}
+  } /*textsub_finish*/
 
 #ifdef TEXTSUB_DEBUG
+/* test program to save text rendering to a PNG file */
+/* this probably will not compile any more */
 
 static int framenum = 0;
+static char *img_name;
 
 struct pngdata {
-	FILE * fp;
-	png_structp png_ptr;
-	png_infop info_ptr;
-	enum {OK,ERROR} status;
+    FILE * fp;
+    png_structp png_ptr;
+    png_infop info_ptr;
+    enum {OK,ERROR} status;
 };
 
-static struct pngdata create_png (char * fname, int image_width, int image_height, int swapped)
-{
+static struct pngdata create_png
+  (
+    const char * fname,
+    int image_width,
+    int image_height,
+    int swapped
+  )
+  /* creates and opens a PNG file and returns a structure ready for writing its contents. */
+  {
     struct pngdata png;
 
-    /*png_structp png_ptr = png_create_write_struct
-       (PNG_LIBPNG_VER_STRING, (png_voidp)user_error_ptr,
-        user_error_fn, user_warning_fn);*/
     //png_byte *row_pointers[image_height];
     png.png_ptr = png_create_write_struct
-       (PNG_LIBPNG_VER_STRING, NULL,
-        NULL, NULL);
+      (
+        /*user_png_ver =*/ PNG_LIBPNG_VER_STRING,
+        /*error_ptr =*/ NULL,
+        /*error_fn =*/ NULL,
+        /*warn_fn =*/ NULL
+      );
     png.info_ptr = png_create_info_struct(png.png_ptr);
-
     if (!png.png_ptr) {
-       if(verbose > 1) fprintf(stderr,"ERR: PNG Failed to init png pointer\n");
-       png.status = ERROR;
-       return png;
+        if(verbose > 1)
+            fprintf(stderr,"ERR: PNG Failed to init png pointer\n");
+        png.status = ERROR;
+        return png;
     }
-
     if (!png.info_ptr) {
-       if(verbose > 1) fprintf(stderr,"ERR: PNG Failed to init png infopointer\n");
-       png_destroy_write_struct(&png.png_ptr,
-         (png_infopp)NULL);
-       png.status = ERROR;
-       return png;
+        if(verbose > 1)
+            fprintf(stderr,"ERR: PNG Failed to init png infopointer\n");
+        png_destroy_write_struct
+          (
+            /*png_ptr_ptr =*/ &png.png_ptr,
+            /*info_ptr_ptr -*/ (png_infopp)NULL
+          );
+        png.status = ERROR;
+        return png;
     }
-
     if (setjmp(png.png_ptr->jmpbuf)) {
-	if(verbose > 1) fprintf(stderr,"ERR: PNG Internal error!\n");
+        if(verbose > 1)
+            fprintf(stderr,"ERR: PNG Internal error!\n");
         png_destroy_write_struct(&png.png_ptr, &png.info_ptr);
         fclose(png.fp);
         png.status = ERROR;
         return png;
     }
 
-    png.fp = fopen (fname, "wb");
+    png.fp = fopen(fname, "wb");
     if (png.fp == NULL) {
-	fprintf(stderr,"ERR: PNG Error opening %s for writing!\n", strerror(errno));
-       	png.status = ERROR;
-       	return png;
+        fprintf(stderr,"ERR: PNG Error opening %s for writing!\n", strerror(errno));
+        png.status = ERROR;
+        return png;
     }
 
-    if(verbose > 1) fprintf(stderr,"INFO: PNG Init IO\n");
+    if(verbose > 1)
+        fprintf(stderr,"INFO: PNG Init IO\n");
     png_init_io(png.png_ptr, png.fp);
 
     /* set the zlib compression level */
     png_set_compression_level(png.png_ptr, 0);
 
+    png_set_IHDR
+      (
+        /*png_ptr =*/ png.png_ptr,
+        /*info_ptr =*/ png.info_ptr,
+        /*width =*/ image_width,
+        /*height =*/ image_height,
+        /*bit_depth =*/ 8,
+        /*color_type =*/ PNG_COLOR_TYPE_RGB,
+        /*interlace_method =*/ PNG_INTERLACE_NONE,
+        /*compression_method =*/ PNG_COMPRESSION_TYPE_DEFAULT,
+        /*filter_method =*/ PNG_FILTER_TYPE_DEFAULT
+      );
 
-    /*png_set_IHDR(png_ptr, info_ptr, width, height,
-       bit_depth, color_type, interlace_type,
-       compression_type, filter_type)*/
-    png_set_IHDR(png.png_ptr, png.info_ptr, image_width, image_height,
-       8, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE,
-       PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
-
-    if(verbose > 1) fprintf(stderr,"INFO: PNG Write Info\n");
+    if(verbose > 1)
+        fprintf(stderr,"INFO: PNG Write Info\n");
     png_write_info(png.png_ptr, png.info_ptr);
 
     if(swapped) {
-    	if(verbose > 1) fprintf(stderr,"INFO: PNG Set BGR Conversion\n");
-    	png_set_bgr(png.png_ptr);
+        if(verbose > 1)
+            fprintf(stderr,"INFO: PNG Set BGR Conversion\n");
+        png_set_bgr(png.png_ptr);
     }
 
     png.status = OK;
     return png;
-}
+  } /*pngdata create_png*/
 
 static uint8_t destroy_png(struct pngdata png) {
 
@@ -320,19 +349,19 @@ char *draw_image(int p_w, int p_h, unsigned char* p_planes,unsigned int p_stride
     struct pngdata png;
     png_byte *row_pointers[p_h];
 
-	img_name=NULL;
+    img_name=NULL;
     snprintf (buf, 100, "%08d.png", ++framenum);
 
     png = create_png(buf, p_w, p_h, 0);
 
     if(png.status){
-	    fprintf(stderr,"ERR: PNG Error in create_png\n");
-	    return NULL;
+        fprintf(stderr,"ERR: PNG Error in create_png\n");
+        return NULL;
     }
 
     if(verbose > 1) fprintf(stderr,"INFO: PNG Creating Row Pointers\n");
     for ( k = 0; k < p_h; k++ )
-	row_pointers[k] = p_planes+p_stride*k;
+    row_pointers[k] = p_planes+p_stride*k;
 
     //png_write_flush(png.png_ptr);
     //png_set_flush(png.png_ptr, nrows);
@@ -341,9 +370,9 @@ char *draw_image(int p_w, int p_h, unsigned char* p_planes,unsigned int p_stride
     png_write_image(png.png_ptr, row_pointers);
 
     destroy_png(png);
-	img_name=malloc(100);
-	strcpy(img_name,buf);
-	return img_name;
+    img_name=malloc(100);
+    strcpy(img_name,buf);
+    return img_name;
 }
 int main(int argc, char **argv)
 {
@@ -384,7 +413,7 @@ int main(int argc, char **argv)
 
   unsigned long pts=0;
   subtitle *last_sub;
-  textsub_subtitle_type *textsub_subtitle;
+  textsub_subtitle_type textsub_subtitle;
   stinfo **spus=NULL;
   stinfo *st=NULL;
   int numspus=0;
@@ -405,21 +434,21 @@ int main(int argc, char **argv)
   }
   if (argc==3)
     textsub_dump_file();
-  last_sub=(&textsub_subs[textsub_subdata->sub_num-1]);
+  last_sub=(&textsub_subdata->subtitles[textsub_subdata->sub_num-1]);
   for ( pts=0;pts<last_sub->end;pts++)
   {
-	textsub_subtitle=textsub_find_sub(pts);
-	if ( textsub_subtitle->image!=NULL)
-	  if (draw_image(movie_width,movie_height,textsub_subtitle->image,movie_width*3)!=NULL)
-	  {
-	    st=malloc(sizeof(stinfo));
+    textsub_subtitle = textsub_find_sub(pts);
+    if ( textsub_subtitle.image!=NULL)
+      if (draw_image(movie_width,movie_height,textsub_subtitle.image,movie_width*3)!=NULL)
+      {
+        st=malloc(sizeof(stinfo));
         memset(st,0,sizeof(stinfo));
-	    strcpy(st->img.fname,img_name);
-	    st->spts=textsub_subtitle->start;
-	    st->sd=(textsub_subtitle->end)-(textsub_subtitle->start);
-	    spus=realloc(spus,(numspus+1)*sizeof(stinfo *));
+        strcpy(st->img.fname,img_name);
+        st->spts=textsub_subtitle.start;
+        st->sd=(textsub_subtitle.end)-(textsub_subtitle.start);
+        spus=realloc(spus,(numspus+1)*sizeof(stinfo *));
         spus[numspus++]=st;
-	  }
+      }
   }
   textsub_finish();
   textsub_statistics();
